# swarmauri/standard/tools/concrete/GithubTool.py

from github import Github, GithubException
from typing import List, Dict, Literal, Any
from pydantic import Field, ConfigDict
from swarmauri.tools.base.ToolBase import ToolBase
from swarmauri.tools.concrete.Parameter import Parameter


class GithubRepoTool(ToolBase):
    version: str = "1.1.0"
    parameters: List[Parameter] = Field(
        default_factory=lambda: [
            Parameter(
                name="action",
                type="string",
                description="The action to perform on the GitHub API, e.g., 'create_repo', 'delete_repo', 'update_repo', and 'get_repo'",
                required=True,
            ),
            Parameter(
                name="repo_name",
                type="string",
                description="The full name of the repository to interact with, e.g. 'owner/repository'.",
                required=False,
            ),
            Parameter(
                name="file_path",
                type="string",
                description="The path to the file in the repository, e.g. 'path/to/file.txt'.",
                required=False,
            ),
        ]
    )
    name: str = "GithubRepoTool"
    description: str = "Interacts with GitHub repositories using PyGithub."
    type: Literal["GithubRepoTool"] = "GithubRepoTool"
    token: str
    model_config = ConfigDict(arbitrary_types_allowed=True)

    def __call__(self, action: str, **kwargs) -> Dict[str, Any]:
        """
        Central method to call various GitHub API actions.

        Args:
            action (str): The action to perform.
            **kwargs: Additional keyword arguments related to the action.

        Returns:
            Dict[str, Any]: The result of the action.
        """
        action_map = {
            "create_repo": self.create_repo,
            "delete_repo": self.delete_repo,
            "get_repo": self.get_repo,
            "list_repos": self.list_repos,
            "update_repo": self.update_repo,
        }

        if action in action_map:
            self._github = Github(self.token)
            return {action: action_map[action](**kwargs)}

        raise ValueError(f"Action '{action}' is not supported.")

    # Repository Management Methods
    def create_repo(self, repo_name: str, private: bool = False) -> str:
        try:
            user = self._github.get_user()
            repo = user.create_repo(repo_name, private=private)
            return f"Repository '{repo_name}' created successfully."
        except GithubException as e:
            return f"Error creating repository: {e}"

    def delete_repo(self, repo_name: str) -> str:
        try:
            user = self._github.get_user()
            repo = user.get_repo(repo_name)
            repo.delete()
            return f"Repository '{repo_name}' deleted successfully."
        except GithubException as e:
            return f"Error deleting repository: {e}"

    def get_repo(self, repo_name: str) -> str:
        try:
            repo = self._github.get_repo(repo_name)
            repo_info = f"Repository: {repo.full_name}\nDescription: {repo.description}\nClone URL: {repo.clone_url}"
            return repo_info
        except GithubException as e:
            return f"Error retrieving repository info: {e}"

    def list_repos(self) -> List[str]:
        try:
            user = self._github.get_user()
            return [repo.full_name for repo in user.get_repos()]
        except GithubException as e:
            return f"Error listing repositories: {e}"

    def update_repo(self, repo_name: str, **kwargs) -> str:
        try:
            repo = self._github.get_repo(repo_name)
            repo.edit(**kwargs)
            return f"Repository '{repo_name}' updated successfully."
        except GithubException as e:
            return f"Error updating repository: {e}"



from swarmauri.utils._lazy_import import _lazy_import

documents_stores_files = [
    ("swarmauri_community.documents_stores.concrete.RedisDocumentStore", "RedisDocumentStore"),
]

for module_name, class_name in documents_stores_files:
    globals()[class_name] = _lazy_import(module_name, class_name)

__all__ = [class_name for _, class_name in documents_stores_files]



import json
from openai import OpenAI, AsyncOpenAI
from typing import List, Dict, Literal, Optional, Iterator, AsyncIterator
import asyncio
from pydantic import Field
from swarmauri_core.typing import SubclassUnion
from swarmauri.messages.base.MessageBase import MessageBase
from swarmauri.messages.concrete.AgentMessage import AgentMessage
from swarmauri.llms.base.LLMBase import LLMBase

from swarmauri.messages.concrete.AgentMessage import UsageData

from swarmauri.utils.duration_manager import DurationManager


class LeptonAIModel(LLMBase):
    """
    Provider resources: https://www.lepton.ai/playground
    """

    api_key: str
    allowed_models: List[str] = [
        "llama2-13b",
        "llama3-1-405b",
        "llama3-1-70b",
        "llama3-1-8b",
        "llama3-70b",
        "llama3-8b",
        "mixtral-8x7b",
        "mistral-7b",
        "nous-hermes-llama2-13b",
        "openchat-3-5",
        "qwen2-72b",
        "toppy-m-7b",
        "wizardlm-2-7b",
        "wizardlm-2-8x22b",
    ]

    name: str = "llama3-8b"
    type: Literal["LeptonAIModel"] = "LeptonAIModel"
    client: OpenAI = Field(default=None, exclude=True)
    async_client: AsyncOpenAI = Field(default=None, exclude=True)

    def __init__(self, **data):
        super().__init__(**data)
        url = f"https://{self.name}.lepton.run/api/v1/"
        self.client = OpenAI(base_url=url, api_key=self.api_key)
        self.async_client = AsyncOpenAI(base_url=url, api_key=self.api_key)

    def _format_messages(
        self, messages: List[SubclassUnion[MessageBase]]
    ) -> List[Dict[str, str]]:
        message_properties = ["content", "role", "name"]
        formatted_messages = [
            message.model_dump(include=message_properties, exclude_none=True)
            for message in messages
        ]
        return formatted_messages

    def _get_system_context(self, messages: List[SubclassUnion[MessageBase]]) -> str:
        system_context = None
        for message in messages:
            if message.role == "system":
                system_context = message.content
        return system_context

    def _prepare_messages(self, conversation):
        formatted_messages = self._format_messages(conversation.history)
        system_context = self._get_system_context(conversation.history)
        if system_context:
            formatted_messages = [
                {"role": "system", "content": system_context},
                formatted_messages[-1],
            ]
        return formatted_messages

    def _prepare_usage_data(
        self,
        usage_data,
        prompt_time: float = 0.0,
        completion_time: float = 0.0,
    ):
        """
        Prepares and extracts usage data and response timing.
        """
        total_time = prompt_time + completion_time

        usage = UsageData(
            prompt_tokens=usage_data.get("prompt_tokens", 0),
            completion_tokens=usage_data.get("completion_tokens", 0),
            total_tokens=usage_data.get("total_tokens", 0),
            prompt_time=prompt_time,
            completion_time=completion_time,
            total_time=total_time,
        )

        return usage

    def predict(
        self,
        conversation,
        temperature: Optional[float] = 0.5,
        max_tokens: Optional[int] = 256,
        top_p: Optional[float] = 0.8,
        stream: Optional[bool] = False,
    ):
        formatted_messages = self._prepare_messages(conversation)

        with DurationManager() as prompt_timer:
            response = self.client.chat.completions.create(
                model=self.name,
                messages=formatted_messages,
                temperature=temperature,
                max_tokens=max_tokens,
                top_p=top_p,
                stream=stream,
            )

        result = json.loads(response.model_dump_json())
        message_content = result["choices"][0]["message"]["content"]
        usage_data = result.get("usage", {})

        usage = self._prepare_usage_data(
            usage_data,
            prompt_timer.duration,
        )

        conversation.add_message(AgentMessage(content=message_content, usage=usage))
        return conversation

    async def apredict(
        self,
        conversation,
        temperature: Optional[float] = 0.5,
        max_tokens: Optional[int] = 256,
        top_p: Optional[float] = 0.8,
    ):
        """Asynchronous version of predict"""
        formatted_messages = self._prepare_messages(conversation)

        with DurationManager() as prompt_timer:
            response = await self.async_client.chat.completions.create(
                model=self.name,
                messages=formatted_messages,
                temperature=temperature,
                max_tokens=max_tokens,
                top_p=top_p,
            )

        result = json.loads(response.model_dump_json())
        message_content = result["choices"][0]["message"]["content"]
        usage_data = result.get("usage", {})

        usage = self._prepare_usage_data(
            usage_data,
            prompt_timer.duration,
        )

        conversation.add_message(AgentMessage(content=message_content, usage=usage))
        return conversation

    def stream(
        self,
        conversation,
        temperature: Optional[float] = 0.5,
        max_tokens: Optional[int] = 256,
        top_p: Optional[float] = 0.8,
    ) -> Iterator[str]:
        """Synchronously stream the response token by token"""
        formatted_messages = self._prepare_messages(conversation)

        with DurationManager() as prompt_timer:
            stream = self.client.chat.completions.create(
                model=self.name,
                messages=formatted_messages,
                temperature=temperature,
                max_tokens=max_tokens,
                top_p=top_p,
                stream=True,
                stream_options={"include_usage": True},
            )

        collected_content = []
        usage_data = {}

        with DurationManager() as completion_timer:
            for chunk in stream:
                if chunk.choices and chunk.choices[0].delta.content:
                    content = chunk.choices[0].delta.content
                    collected_content.append(content)
                    yield content

                if hasattr(chunk, "usage") and chunk.usage is not None:
                    usage_data = chunk.usage

        full_content = "".join(collected_content)
        usage = self._prepare_usage_data(
            usage_data.model_dump(),
            prompt_timer.duration,
            completion_timer.duration,
        )

        conversation.add_message(AgentMessage(content=full_content, usage=usage))

    async def astream(
        self,
        conversation,
        temperature: Optional[float] = 0.5,
        max_tokens: Optional[int] = 256,
        top_p: Optional[float] = 0.8,
    ) -> AsyncIterator[str]:
        """Asynchronously stream the response token by token"""
        formatted_messages = self._prepare_messages(conversation)

        with DurationManager() as prompt_timer:
            stream = await self.async_client.chat.completions.create(
                model=self.name,
                messages=formatted_messages,
                temperature=temperature,
                max_tokens=max_tokens,
                top_p=top_p,
                stream=True,
                stream_options={"include_usage": True},
            )

        usage_data = {}
        collected_content = []

        with DurationManager() as completion_timer:
            async for chunk in stream:
                if chunk.choices and chunk.choices[0].delta.content:
                    content = chunk.choices[0].delta.content
                    collected_content.append(content)
                    yield content

                if hasattr(chunk, "usage") and chunk.usage is not None:
                    usage_data = chunk.usage

        full_content = "".join(collected_content)

        usage = self._prepare_usage_data(
            usage_data.model_dump(),
            prompt_timer.duration,
            completion_timer.duration,
        )
        conversation.add_message(AgentMessage(content=full_content, usage=usage))

    def batch(
        self,
        conversations: List,
        temperature: Optional[float] = 0.5,
        max_tokens: Optional[int] = 256,
        top_p: Optional[float] = 0.8,
    ) -> List:
        """Synchronously process multiple conversations"""
        return [
            self.predict(
                conv,
                temperature=temperature,
                max_tokens=max_tokens,
                top_p=top_p,
            )
            for conv in conversations
        ]

    async def abatch(
        self,
        conversations: List,
        temperature: Optional[float] = 0.5,
        max_tokens: Optional[int] = 256,
        top_p: Optional[float] = 0.8,
        max_concurrent: int = 5,
    ) -> List:
        """Process multiple conversations in parallel with controlled concurrency"""
        semaphore = asyncio.Semaphore(max_concurrent)

        async def process_conversation(conv):
            async with semaphore:
                return await self.apredict(
                    conv,
                    temperature=temperature,
                    max_tokens=max_tokens,
                    top_p=top_p,
                )

        tasks = [process_conversation(conv) for conv in conversations]
        return await asyncio.gather(*tasks)



from typing import List, Optional
from ....standard.document_stores.base.DocumentStoreBase import DocumentStoreBase
from ....core.documents.IDocument import IDocument
import redis
import json
from redis.commands.search.field import TextField, NumericField, TagField
from redis.commands.search.indexDefinition import IndexDefinition, IndexType


class RedisDocumentStore(DocumentStoreBase):
    def __init__(self, host, password, port, db):
        """Store connection details without initializing the Redis client."""
        self._host = host
        self._password = password
        self._port = port
        self._db = db
        self._redis_client = None  # Delayed initialization

    @property
    def redis_client(self):
        """Lazily initialize and return the Redis client using a factory method."""
        if self._redis_client is None:
            print('here')
            self._redis_client = redis.Redis(host=self._host, 
                                             password=self._password, 
                                             port=self._port, 
                                             db=self._db)
            print('there')
        return self._redis_client

    def add_document(self, document: IDocument) -> None:
        
        data = document.as_dict()
        doc_id = data['id'] 
        del data['id']
        self.redis_client.json().set(doc_id, '$', json.dumps(data))

    def add_documents(self, documents: List[IDocument]) -> None:
        with self.redis_client.pipeline() as pipe:
            for document in documents:
                pipe.set(document.doc_id, document)
            pipe.execute()

    def get_document(self, doc_id: str) -> Optional[IDocument]:
        result = self.redis_client.json().get(doc_id)
        if result:
            return json.loads(result)
        return None

    def get_all_documents(self) -> List[IDocument]:
        keys = self.redis_client.keys('*')
        documents = []
        for key in keys:
            document_data = self.redis_client.get(key)
            if document_data:
                documents.append(json.loads(document_data))
        return documents

    def update_document(self, doc_id: str, updated_document: IDocument) -> None:
        self.add_document(updated_document)

    def delete_document(self, doc_id: str) -> None:
        self.redis_client.delete(doc_id)
    
    def __getstate__(self):
        """Return the object state for serialization, excluding the Redis client."""
        state = self.__dict__.copy()
        state['_redis_client'] = None  # Exclude Redis client from serialization
        return state

    def __setstate__(self, state):
        """Restore the object state after serialization, reinitializing the Redis client."""
        self.__dict__.update(state)


import base64
import json
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from google.oauth2 import service_account
from googleapiclient.discovery import build, Resource
from swarmauri.tools.base.ToolBase import ToolBase
from swarmauri.tools.concrete.Parameter import Parameter
from typing import List, Dict, Literal
from pydantic import Field


class GmailSendTool(ToolBase):
    SCOPES: List[str] = ["https://www.googleapis.com/auth/gmail.send"]

    version: str = "1.0.0"
    parameters: List[Parameter] = Field(
        default_factory=lambda: [
            Parameter(
                name="recipients",
                type="string",
                description="The email addresses of the recipients, separated by commas",
                required=True,
            ),
            Parameter(
                name="subject",
                type="string",
                description="The subject of the email",
                required=True,
            ),
            Parameter(
                name="htmlMsg",
                type="string",
                description="The HTML message to be sent as the email body",
                required=True,
            ),
        ]
    )

    name: str = "GmailSendTool"
    description: str = "Sends an email using the Gmail API."
    type: Literal["GmailSendTool"] = "GmailSendTool"

    credentials_path: str
    sender_email: str

    def authenticate(self) -> Resource:
        """
        Authenticates the user and creates a Gmail API service for sending emails.
        This method returns the service instance, without saving it as an attribute.
        """
        credentials = service_account.Credentials.from_service_account_file(
            self.credentials_path, scopes=self.SCOPES
        )
        delegated_credentials = credentials.with_subject(self.sender_email)
        return build("gmail", "v1", credentials=delegated_credentials)

    def create_message(
        self, to: str, subject: str, message_text: str
    ) -> Dict[str, str]:
        """
        Create a MIMEText message for sending an email.

        Parameters:
        to (str): The email address of the recipient.
        subject (str): The subject of the email.
        message_text (str): The HTML body of the email.

        Returns:
        Dict[str, str]: The created MIMEText message in a format suitable for Gmail API.
        """
        message = MIMEMultipart("alternative")
        message["from"] = self.sender_email
        message["to"] = to
        message["subject"] = subject
        mime_text = MIMEText(message_text, "html")
        message.attach(mime_text)
        raw_message = base64.urlsafe_b64encode(
            message.as_string().encode("utf-8")
        ).decode("utf-8")
        return {"raw": raw_message}

    def __call__(self, recipients: str, subject: str, htmlMsg: str) -> Dict[str, str]:
        """
        Sends an email to the specified recipients with the given subject and HTML message.

        Parameters:
            recipients (str): The email address of the recipients, separated by commas.
            subject (str): The subject of the email.
            htmlMsg (str): The HTML content of the email body.

        Returns:
            Dict[str, str]: A message indicating the status of the email sending process.
        """
        service = (
            self.authenticate()
        )  # Authenticate within this function and do not store in the object state
        try:
            message = self.create_message(recipients, subject, htmlMsg)
            service.users().messages().send(userId="me", body=message).execute()
            return {"success": f"Email sent successfully to {recipients}"}
        except Exception as e:
            return {"error": f"An error occurred in sending the email: {str(e)}"}



from typing import List, Union, Literal, Optional
from pydantic import BaseModel, PrivateAttr
import uuid as ud
import weaviate
from weaviate.classes.init import Auth
from weaviate.util import generate_uuid5
from weaviate.classes.query import MetadataQuery

from swarmauri.documents.concrete.Document import Document
from swarmauri_community.embeddings.concrete.Doc2VecEmbedding import Doc2VecEmbedding
from swarmauri.vectors.concrete.Vector import Vector

from swarmauri.vector_stores.base.VectorStoreBase import VectorStoreBase
from swarmauri.vector_stores.base.VectorStoreRetrieveMixin import VectorStoreRetrieveMixin
from swarmauri.vector_stores.base.VectorStoreSaveLoadMixin import VectorStoreSaveLoadMixin
from swarmauri.vector_stores.base.VectorStoreCloudMixin import VectorStoreCloudMixin


class CloudWeaviateVectorStore(VectorStoreSaveLoadMixin, VectorStoreRetrieveMixin, VectorStoreBase, VectorStoreCloudMixin):
    type: Literal["CloudWeaviateVectorStore"] = "CloudWeaviateVectorStore"
    

    # Private attributes
    _client: Optional[weaviate.Client] = PrivateAttr(default=None)
    _embedder: Doc2VecEmbedding = PrivateAttr(default=None)
    _namespace_uuid: ud.UUID = PrivateAttr(default_factory=ud.uuid4)

    def __init__(self, **data):
        super().__init__(**data)

        # Initialize the vectorizer and Weaviate client
        self._embedder = Doc2VecEmbedding(vector_size=self.vector_size)
        # self._initialize_client()

    def connect(self, **kwargs):
        """
        Initialize the Weaviate client.
        """
        if self._client is None:
            self._client = weaviate.connect_to_weaviate_cloud(
                cluster_url=self.url,
                auth_credentials=Auth.api_key(self.api_key),
                headers=kwargs.get("headers", {})
            )
    
    def disconnect(self) -> None:
        """
        Disconnects from the Qdrant cloud vector store.
        """
        if self.client is not None:
            self.client = None

    def add_document(self, document: Document) -> None:
        """
        Add a single document to the vector store.

        :param document: Document to add
        """
        try:
            collection = self._client.collections.get(self.collection_name)

            # Generate or use existing embedding
            embedding = document.embedding or self._embedder.fit_transform([document.content])[0]

            data_object = {
                "content": document.content,
                "metadata": document.metadata,
            }

            # Generate UUID for document
            uuid = (
                str(ud.uuid5(self._namespace_uuid, document.id))
                if document.id
                else generate_uuid5(data_object)
            )

            collection.data.insert(
                properties=data_object,
                vector=embedding.value,
                uuid=uuid,
            )

            print(f"Document '{document.id}' added to Weaviate.")
        except Exception as e:
            print(f"Error adding document '{document.id}': {e}")
            raise

    def add_documents(self, documents: List[Document]) -> None:
        """
        Add multiple documents to the vector store.

        :param documents: List of documents to add
        """
        try:
            for document in documents:
                self.add_document(document)

            print(f"{len(documents)} documents added to Weaviate.")
        except Exception as e:
            print(f"Error adding documents: {e}")
            raise

    def get_document(self, id: str) -> Union[Document, None]:
        """
        Retrieve a document by its ID.

        :param id: Document ID
        :return: Document object or None if not found
        """
        try:
            collection = self._client.collections.get(self.collection_name)

            result = collection.query.fetch_object_by_id(ud.uuid5(self._namespace_uuid, id))

            if result:
                return Document(
                    id=id,
                    content=result.properties["content"],
                    metadata=result.properties["metadata"],
                )
            return None
        except Exception as e:
            print(f"Error retrieving document '{id}': {e}")
            return None

    def get_all_documents(self) -> List[Document]:
        """
        Retrieve all documents from the vector store.

        :return: List of Document objects
        """
        try:
            collection = self._client.collections.get(self.collection_name)
            # return collection
            documents = [
                Document(
                    content=item.properties["content"],
                    metadata=item.properties["metadata"],
                    embedding=Vector(value=list(item.vector.values())[0]),
                )
                for item in collection.iterator(include_vector=True)
            ]
            return documents
        except Exception as e:
            print(f"Error retrieving all documents: {e}")
            return []

    def delete_document(self, id: str) -> None:
        """
        Delete a document by its ID.

        :param id: Document ID
        """
        try:
            collection = self._client.collections.get(self.collection_name)
            collection.data.delete_by_id(ud.uuid5(self._namespace_uuid, id))
            print(f"Document '{id}' has been deleted from Weaviate.")
        except Exception as e:
            print(f"Error deleting document '{id}': {e}")
            raise

    def update_document(self, id: str, document: Document) -> None:
        """
        Update an existing document.

        :param id: Document ID
        :param updated_document: Document object with updated data
        """
        self.delete_document(id)
        self.add_document(document)

    def retrieve(self, query: str, top_k: int = 5) -> List[Document]:
        """
        Retrieve the top_k most relevant documents based on the given query.

        :param query: Query string
        :param top_k: Number of top similar documents to retrieve
        :return: List of Document objects
        """
        try:
            collection = self._client.collections.get(self.collection_name)
            query_vector = self._embedder.infer_vector(query)
            response = collection.query.near_vector(
                near_vector=query_vector.value,
                limit=top_k,
                return_metadata=MetadataQuery(distance=True),
            )

            documents = [
                Document(
                    # id=res.id,
                    content=res.properties["content"],
                    metadata=res.properties["metadata"],
                )
                for res in response.objects
            ]
            return documents
        except Exception as e:
            print(f"Error retrieving documents for query '{query}': {e}")
            return []

    def close(self):
        """
        Close the connection to the Weaviate server.
        """
        if self._client:
            self._client.close()

    def model_dump_json(self, *args, **kwargs) -> str:
        # Call the disconnect method before serialization
        self.disconnect()

        # Now proceed with the usual JSON serialization
        return super().model_dump_json(*args, **kwargs)

    
    def __del__(self):
        self.close()


# swarmauri/standard/tools/concrete/GithubTool.py

from github import Github, GithubException
from typing import List, Dict, Literal, Optional, Any
from pydantic import Field, ConfigDict
from swarmauri.tools.base.ToolBase import ToolBase
from swarmauri.tools.concrete.Parameter import Parameter


class GithubCommitTool(ToolBase):
    version: str = "1.1.0"
    parameters: List[Parameter] = Field(
        default_factory=lambda: [
            Parameter(
                name="action",
                type="string",
                description="The action to perform on the GitHub API, e.g., 'create_repo', 'delete_repo', 'create_issue', etc.",
                required=True,
            ),
            Parameter(
                name="repo_name",
                type="string",
                description="The full name of the repository to interact with, e.g. 'owner/repository'.",
                required=True,
            ),
            Parameter(
                name="file_path",
                type="string",
                description="The path to the file in the repository, e.g. 'path/to/file.txt'.",
                required=False,
            ),
            Parameter(
                name="message",
                type="string",
                description=".",
                required=False,
            ),
            Parameter(
                name="content",
                type="string",
                description="The name of the branch to interact with.",
                required=False,
            ),
            Parameter(
                name="branch_name",
                type="string",
                description="The name of the branch to interact with.",
                required=False,
            ),
            Parameter(
                name="sha",
                type="string",
                description="The sha of the commit to interact with.",
                required=False,
            ),
            Parameter(
                name="base",
                type="string",
                description="The base of the commit to interact with.",
                required=False,
            ),
            Parameter(
                name="head",
                type="string",
                description="The head of the commit to interact with.",
                required=False,
            ),
        ]
    )
    name: str = "GithubCommitTool"
    description: str = "Interacts with GitHub repositories using PyGithub to submit commits."
    type: Literal["GithubCommitTool"] = "GithubCommitTool"
    token: str
    model_config = ConfigDict(arbitrary_types_allowed=True)

    def __call__(self, action: str, **kwargs) -> Dict[str, Any]:
        """
        Central method to call various GitHub API actions.

        Args:
            action (str): The action to perform.
            **kwargs: Additional keyword arguments related to the action.

        Returns:
            Dict[str, Any]: The result of the action.
        """
        action_map = {
            "create_commit": self.create_commit,
            "list_commits": self.list_commits,
            "get_commit": self.get_commit,
            "compare_commits": self.compare_commits
        }

        if action in action_map:
            self._github = Github(self.token)
            return {action: action_map[action](**kwargs)}

        raise ValueError(f"Action '{action}' is not supported.")


    # Commit Management Methods
    def create_commit(
        self,
        repo_name: str,
        file_path: str,
        message: str,
        content: str,
        branch: str = "main",
    ) -> str:
        try:
            repo = self._github.get_repo(repo_name)
            repo.create_file(path=file_path, message=message, content=content, branch=branch)
            return f"Commit created successfully at {file_path}."
        except GithubException as e:
            return f"Error creating commit: {e}"

    def list_commits(self, repo_name: str) -> List[str]:
        try:
            repo = self._github.get_repo(repo_name)
            return [commit.commit.message for commit in repo.get_commits()]
        except GithubException as e:
            return f"Error listing commits: {e}"

    def get_commit(self, repo_name: str, sha: str) -> str:
        try:
            repo = self._github.get_repo(repo_name)
            commit = repo.get_commit(sha=sha)
            return f"Commit {commit.sha}: {commit.commit.message}"
        except GithubException as e:
            return f"Error retrieving commit: {e}"

    def compare_commits(self, repo_name: str, base: str, head: str) -> str:
        try:
            repo = self._github.get_repo(repo_name)
            comparison = repo.compare(base, head)
            return f"Comparison from {base} to {head}:\n{comparison.diff_url}"
        except GithubException as e:
            return f"Error comparing commits: {e}"




from swarmauri.utils._lazy_import import _lazy_import

llms_files = [
    ("swarmauri_community.llms.concrete.LeptonAIImgGenModel", "LeptonAIImgGenModel"),
    ("swarmauri_community.llms.concrete.LeptonAIModel", "LeptonAIModel"),
    ("swarmauri_community.llms.concrete.PytesseractImg2TextModel", "PytesseractImg2TextModel"),
]

for module_name, class_name in llms_files:
    globals()[class_name] = _lazy_import(module_name, class_name)

__all__ = [class_name for _, class_name in llms_files]
